; table driven CRC32 for Z80, Will Sowerbutts 2014-06-04.
; the data table could be calculated at runtime to reduce executable size.

    .module crc32
    .globl _crc32

    .area _CODE

_crc32:
        push ix ; save IX
        push iy ; save IY -- do we need to? why not.

        ld iy, #0
        add iy, sp

        ; get buffer address from sp+6 into IX
        ld c, 6(iy)
        ld b, 7(iy)
        ld ix, #0
        add ix, bc

        ; get length from sp+8 into BC
        ld c, 8(iy)
        ld b, 9(iy)

        ; get input CRC from sp+10 into HLDE
        ld e, 10(iy) ; lsb
        ld d, 11(iy)
        ld l, 12(iy)
        ld h, 13(iy) ; msb

; Enter here with IX=addr, BC=num, HLDE=crc
nextbyte:
        push bc         ; save remaining byte count
        
        ; compute LSB ^ input byte -- table index
        ld a, (ix)
        xor e
        ld b, #0
        ld c, a         ; store index in BC

        ; lookup address in table: 4 x index
        ld iy, #crc32_table
        add iy, bc
        add iy, bc
        add iy, bc
        add iy, bc

        ; update the CRC using table values
        ld a, (iy)
        xor d
        ld e, a
        
        ld a, 1(iy)
        xor l
        ld d, a

        ld a, 2(iy)
        xor h
        ld l, a

        ld h, 3(iy)

        pop bc          ; recover byte count

        ; next byte
        dec bc
        inc ix
        ld a, b
        or c
        jr nz, nextbyte

        ; new CRC is in DEHL
alldone:
        ex de, hl ; return with CRC in HLDE (not DEHL) -- why compiler, WHY??
        pop iy
        pop ix
        ret

    .area _DATA

; precomputed CRC32 table
crc32_table:
    .db 0x00, 0x00, 0x00, 0x00, 0x96, 0x30, 0x07, 0x77
    .db 0x2c, 0x61, 0x0e, 0xee, 0xba, 0x51, 0x09, 0x99
    .db 0x19, 0xc4, 0x6d, 0x07, 0x8f, 0xf4, 0x6a, 0x70
    .db 0x35, 0xa5, 0x63, 0xe9, 0xa3, 0x95, 0x64, 0x9e
    .db 0x32, 0x88, 0xdb, 0x0e, 0xa4, 0xb8, 0xdc, 0x79
    .db 0x1e, 0xe9, 0xd5, 0xe0, 0x88, 0xd9, 0xd2, 0x97
    .db 0x2b, 0x4c, 0xb6, 0x09, 0xbd, 0x7c, 0xb1, 0x7e
    .db 0x07, 0x2d, 0xb8, 0xe7, 0x91, 0x1d, 0xbf, 0x90
    .db 0x64, 0x10, 0xb7, 0x1d, 0xf2, 0x20, 0xb0, 0x6a
    .db 0x48, 0x71, 0xb9, 0xf3, 0xde, 0x41, 0xbe, 0x84
    .db 0x7d, 0xd4, 0xda, 0x1a, 0xeb, 0xe4, 0xdd, 0x6d
    .db 0x51, 0xb5, 0xd4, 0xf4, 0xc7, 0x85, 0xd3, 0x83
    .db 0x56, 0x98, 0x6c, 0x13, 0xc0, 0xa8, 0x6b, 0x64
    .db 0x7a, 0xf9, 0x62, 0xfd, 0xec, 0xc9, 0x65, 0x8a
    .db 0x4f, 0x5c, 0x01, 0x14, 0xd9, 0x6c, 0x06, 0x63
    .db 0x63, 0x3d, 0x0f, 0xfa, 0xf5, 0x0d, 0x08, 0x8d
    .db 0xc8, 0x20, 0x6e, 0x3b, 0x5e, 0x10, 0x69, 0x4c
    .db 0xe4, 0x41, 0x60, 0xd5, 0x72, 0x71, 0x67, 0xa2
    .db 0xd1, 0xe4, 0x03, 0x3c, 0x47, 0xd4, 0x04, 0x4b
    .db 0xfd, 0x85, 0x0d, 0xd2, 0x6b, 0xb5, 0x0a, 0xa5
    .db 0xfa, 0xa8, 0xb5, 0x35, 0x6c, 0x98, 0xb2, 0x42
    .db 0xd6, 0xc9, 0xbb, 0xdb, 0x40, 0xf9, 0xbc, 0xac
    .db 0xe3, 0x6c, 0xd8, 0x32, 0x75, 0x5c, 0xdf, 0x45
    .db 0xcf, 0x0d, 0xd6, 0xdc, 0x59, 0x3d, 0xd1, 0xab
    .db 0xac, 0x30, 0xd9, 0x26, 0x3a, 0x00, 0xde, 0x51
    .db 0x80, 0x51, 0xd7, 0xc8, 0x16, 0x61, 0xd0, 0xbf
    .db 0xb5, 0xf4, 0xb4, 0x21, 0x23, 0xc4, 0xb3, 0x56
    .db 0x99, 0x95, 0xba, 0xcf, 0x0f, 0xa5, 0xbd, 0xb8
    .db 0x9e, 0xb8, 0x02, 0x28, 0x08, 0x88, 0x05, 0x5f
    .db 0xb2, 0xd9, 0x0c, 0xc6, 0x24, 0xe9, 0x0b, 0xb1
    .db 0x87, 0x7c, 0x6f, 0x2f, 0x11, 0x4c, 0x68, 0x58
    .db 0xab, 0x1d, 0x61, 0xc1, 0x3d, 0x2d, 0x66, 0xb6
    .db 0x90, 0x41, 0xdc, 0x76, 0x06, 0x71, 0xdb, 0x01
    .db 0xbc, 0x20, 0xd2, 0x98, 0x2a, 0x10, 0xd5, 0xef
    .db 0x89, 0x85, 0xb1, 0x71, 0x1f, 0xb5, 0xb6, 0x06
    .db 0xa5, 0xe4, 0xbf, 0x9f, 0x33, 0xd4, 0xb8, 0xe8
    .db 0xa2, 0xc9, 0x07, 0x78, 0x34, 0xf9, 0x00, 0x0f
    .db 0x8e, 0xa8, 0x09, 0x96, 0x18, 0x98, 0x0e, 0xe1
    .db 0xbb, 0x0d, 0x6a, 0x7f, 0x2d, 0x3d, 0x6d, 0x08
    .db 0x97, 0x6c, 0x64, 0x91, 0x01, 0x5c, 0x63, 0xe6
    .db 0xf4, 0x51, 0x6b, 0x6b, 0x62, 0x61, 0x6c, 0x1c
    .db 0xd8, 0x30, 0x65, 0x85, 0x4e, 0x00, 0x62, 0xf2
    .db 0xed, 0x95, 0x06, 0x6c, 0x7b, 0xa5, 0x01, 0x1b
    .db 0xc1, 0xf4, 0x08, 0x82, 0x57, 0xc4, 0x0f, 0xf5
    .db 0xc6, 0xd9, 0xb0, 0x65, 0x50, 0xe9, 0xb7, 0x12
    .db 0xea, 0xb8, 0xbe, 0x8b, 0x7c, 0x88, 0xb9, 0xfc
    .db 0xdf, 0x1d, 0xdd, 0x62, 0x49, 0x2d, 0xda, 0x15
    .db 0xf3, 0x7c, 0xd3, 0x8c, 0x65, 0x4c, 0xd4, 0xfb
    .db 0x58, 0x61, 0xb2, 0x4d, 0xce, 0x51, 0xb5, 0x3a
    .db 0x74, 0x00, 0xbc, 0xa3, 0xe2, 0x30, 0xbb, 0xd4
    .db 0x41, 0xa5, 0xdf, 0x4a, 0xd7, 0x95, 0xd8, 0x3d
    .db 0x6d, 0xc4, 0xd1, 0xa4, 0xfb, 0xf4, 0xd6, 0xd3
    .db 0x6a, 0xe9, 0x69, 0x43, 0xfc, 0xd9, 0x6e, 0x34
    .db 0x46, 0x88, 0x67, 0xad, 0xd0, 0xb8, 0x60, 0xda
    .db 0x73, 0x2d, 0x04, 0x44, 0xe5, 0x1d, 0x03, 0x33
    .db 0x5f, 0x4c, 0x0a, 0xaa, 0xc9, 0x7c, 0x0d, 0xdd
    .db 0x3c, 0x71, 0x05, 0x50, 0xaa, 0x41, 0x02, 0x27
    .db 0x10, 0x10, 0x0b, 0xbe, 0x86, 0x20, 0x0c, 0xc9
    .db 0x25, 0xb5, 0x68, 0x57, 0xb3, 0x85, 0x6f, 0x20
    .db 0x09, 0xd4, 0x66, 0xb9, 0x9f, 0xe4, 0x61, 0xce
    .db 0x0e, 0xf9, 0xde, 0x5e, 0x98, 0xc9, 0xd9, 0x29
    .db 0x22, 0x98, 0xd0, 0xb0, 0xb4, 0xa8, 0xd7, 0xc7
    .db 0x17, 0x3d, 0xb3, 0x59, 0x81, 0x0d, 0xb4, 0x2e
    .db 0x3b, 0x5c, 0xbd, 0xb7, 0xad, 0x6c, 0xba, 0xc0
    .db 0x20, 0x83, 0xb8, 0xed, 0xb6, 0xb3, 0xbf, 0x9a
    .db 0x0c, 0xe2, 0xb6, 0x03, 0x9a, 0xd2, 0xb1, 0x74
    .db 0x39, 0x47, 0xd5, 0xea, 0xaf, 0x77, 0xd2, 0x9d
    .db 0x15, 0x26, 0xdb, 0x04, 0x83, 0x16, 0xdc, 0x73
    .db 0x12, 0x0b, 0x63, 0xe3, 0x84, 0x3b, 0x64, 0x94
    .db 0x3e, 0x6a, 0x6d, 0x0d, 0xa8, 0x5a, 0x6a, 0x7a
    .db 0x0b, 0xcf, 0x0e, 0xe4, 0x9d, 0xff, 0x09, 0x93
    .db 0x27, 0xae, 0x00, 0x0a, 0xb1, 0x9e, 0x07, 0x7d
    .db 0x44, 0x93, 0x0f, 0xf0, 0xd2, 0xa3, 0x08, 0x87
    .db 0x68, 0xf2, 0x01, 0x1e, 0xfe, 0xc2, 0x06, 0x69
    .db 0x5d, 0x57, 0x62, 0xf7, 0xcb, 0x67, 0x65, 0x80
    .db 0x71, 0x36, 0x6c, 0x19, 0xe7, 0x06, 0x6b, 0x6e
    .db 0x76, 0x1b, 0xd4, 0xfe, 0xe0, 0x2b, 0xd3, 0x89
    .db 0x5a, 0x7a, 0xda, 0x10, 0xcc, 0x4a, 0xdd, 0x67
    .db 0x6f, 0xdf, 0xb9, 0xf9, 0xf9, 0xef, 0xbe, 0x8e
    .db 0x43, 0xbe, 0xb7, 0x17, 0xd5, 0x8e, 0xb0, 0x60
    .db 0xe8, 0xa3, 0xd6, 0xd6, 0x7e, 0x93, 0xd1, 0xa1
    .db 0xc4, 0xc2, 0xd8, 0x38, 0x52, 0xf2, 0xdf, 0x4f
    .db 0xf1, 0x67, 0xbb, 0xd1, 0x67, 0x57, 0xbc, 0xa6
    .db 0xdd, 0x06, 0xb5, 0x3f, 0x4b, 0x36, 0xb2, 0x48
    .db 0xda, 0x2b, 0x0d, 0xd8, 0x4c, 0x1b, 0x0a, 0xaf
    .db 0xf6, 0x4a, 0x03, 0x36, 0x60, 0x7a, 0x04, 0x41
    .db 0xc3, 0xef, 0x60, 0xdf, 0x55, 0xdf, 0x67, 0xa8
    .db 0xef, 0x8e, 0x6e, 0x31, 0x79, 0xbe, 0x69, 0x46
    .db 0x8c, 0xb3, 0x61, 0xcb, 0x1a, 0x83, 0x66, 0xbc
    .db 0xa0, 0xd2, 0x6f, 0x25, 0x36, 0xe2, 0x68, 0x52
    .db 0x95, 0x77, 0x0c, 0xcc, 0x03, 0x47, 0x0b, 0xbb
    .db 0xb9, 0x16, 0x02, 0x22, 0x2f, 0x26, 0x05, 0x55
    .db 0xbe, 0x3b, 0xba, 0xc5, 0x28, 0x0b, 0xbd, 0xb2
    .db 0x92, 0x5a, 0xb4, 0x2b, 0x04, 0x6a, 0xb3, 0x5c
    .db 0xa7, 0xff, 0xd7, 0xc2, 0x31, 0xcf, 0xd0, 0xb5
    .db 0x8b, 0x9e, 0xd9, 0x2c, 0x1d, 0xae, 0xde, 0x5b
    .db 0xb0, 0xc2, 0x64, 0x9b, 0x26, 0xf2, 0x63, 0xec
    .db 0x9c, 0xa3, 0x6a, 0x75, 0x0a, 0x93, 0x6d, 0x02
    .db 0xa9, 0x06, 0x09, 0x9c, 0x3f, 0x36, 0x0e, 0xeb
    .db 0x85, 0x67, 0x07, 0x72, 0x13, 0x57, 0x00, 0x05
    .db 0x82, 0x4a, 0xbf, 0x95, 0x14, 0x7a, 0xb8, 0xe2
    .db 0xae, 0x2b, 0xb1, 0x7b, 0x38, 0x1b, 0xb6, 0x0c
    .db 0x9b, 0x8e, 0xd2, 0x92, 0x0d, 0xbe, 0xd5, 0xe5
    .db 0xb7, 0xef, 0xdc, 0x7c, 0x21, 0xdf, 0xdb, 0x0b
    .db 0xd4, 0xd2, 0xd3, 0x86, 0x42, 0xe2, 0xd4, 0xf1
    .db 0xf8, 0xb3, 0xdd, 0x68, 0x6e, 0x83, 0xda, 0x1f
    .db 0xcd, 0x16, 0xbe, 0x81, 0x5b, 0x26, 0xb9, 0xf6
    .db 0xe1, 0x77, 0xb0, 0x6f, 0x77, 0x47, 0xb7, 0x18
    .db 0xe6, 0x5a, 0x08, 0x88, 0x70, 0x6a, 0x0f, 0xff
    .db 0xca, 0x3b, 0x06, 0x66, 0x5c, 0x0b, 0x01, 0x11
    .db 0xff, 0x9e, 0x65, 0x8f, 0x69, 0xae, 0x62, 0xf8
    .db 0xd3, 0xff, 0x6b, 0x61, 0x45, 0xcf, 0x6c, 0x16
    .db 0x78, 0xe2, 0x0a, 0xa0, 0xee, 0xd2, 0x0d, 0xd7
    .db 0x54, 0x83, 0x04, 0x4e, 0xc2, 0xb3, 0x03, 0x39
    .db 0x61, 0x26, 0x67, 0xa7, 0xf7, 0x16, 0x60, 0xd0
    .db 0x4d, 0x47, 0x69, 0x49, 0xdb, 0x77, 0x6e, 0x3e
    .db 0x4a, 0x6a, 0xd1, 0xae, 0xdc, 0x5a, 0xd6, 0xd9
    .db 0x66, 0x0b, 0xdf, 0x40, 0xf0, 0x3b, 0xd8, 0x37
    .db 0x53, 0xae, 0xbc, 0xa9, 0xc5, 0x9e, 0xbb, 0xde
    .db 0x7f, 0xcf, 0xb2, 0x47, 0xe9, 0xff, 0xb5, 0x30
    .db 0x1c, 0xf2, 0xbd, 0xbd, 0x8a, 0xc2, 0xba, 0xca
    .db 0x30, 0x93, 0xb3, 0x53, 0xa6, 0xa3, 0xb4, 0x24
    .db 0x05, 0x36, 0xd0, 0xba, 0x93, 0x06, 0xd7, 0xcd
    .db 0x29, 0x57, 0xde, 0x54, 0xbf, 0x67, 0xd9, 0x23
    .db 0x2e, 0x7a, 0x66, 0xb3, 0xb8, 0x4a, 0x61, 0xc4
    .db 0x02, 0x1b, 0x68, 0x5d, 0x94, 0x2b, 0x6f, 0x2a
    .db 0x37, 0xbe, 0x0b, 0xb4, 0xa1, 0x8e, 0x0c, 0xc3
    .db 0x1b, 0xdf, 0x05, 0x5a, 0x8d, 0xef, 0x02, 0x2d
